pub fn score(bs: &[u8]) -> i64 {
    for b in bs.iter() {
        if *b >= 128 {
            return -1000;
        }
    }

    let Ok(string) = std::str::from_utf8(bs) else {
        return -1000
    };

    let mut score = 0;

    for c in string.chars() {
        if "aeiou".contains(c) { score += 10 }
        else if c.is_ascii_lowercase() { score += 5 }
        else if c.is_ascii_uppercase() { score += 2 }
        else if c.is_ascii_digit() { score += 2 }
        else if c.is_whitespace() { score += 1 }
        else if c.is_ascii_punctuation() { score += 1 }
        else if c.is_control() {
            score -= 10
        }
        else {
            score -= 20 // wtf?
        }
    }

    for _ in string.match_indices("the ") { score += 40 }
    for _ in string.match_indices("The ") { score += 50 }
    for _ in string.match_indices("be ") { score += 40 }
    for _ in string.match_indices("Be ") { score += 50 }
    for _ in string.match_indices("to ") { score += 40 }
    for _ in string.match_indices("To ") { score += 50 }
    for _ in string.match_indices("of ") { score += 40 }
    for _ in string.match_indices("Of ") { score += 50 }
    for _ in string.match_indices("and ") { score += 40 }
    for _ in string.match_indices("And ") { score += 50 }
    for _ in string.match_indices("have ") { score += 40 }
    for _ in string.match_indices("Have ") { score += 50 }
    for _ in string.match_indices("it ") { score += 40 }
    for _ in string.match_indices("It ") { score += 50 }

    score
}

pub struct Bhattacharyya {
    pub weights: [f32; 256]
}

impl Bhattacharyya {
    pub fn compute(text: &[u8]) -> Self {
        let mut counts: [usize; 256] = [0; 256];
        let total = text.len();
        if total == 0 { return Bhattacharyya { weights: [1.0/256.0; 256] } }

        for c in text {
            counts[*c as usize] += 1;
        }
        let mut weights = [0.0; 256];
        for i in 0..weights.len() {
            weights[i] = counts[i] as f32/total as f32;
        }
        return Bhattacharyya { weights }

    }
    pub fn score(&self, other: &Bhattacharyya) -> f32 {
        let mut sum = 0.0;
        for i in 0..self.weights.len() {
            sum += (self.weights[i] * other.weights[i]).sqrt()
        };
        return sum  // bhattacharyya coefficient (not distance)
        // return -sum.ln();
    }
}